@@ -31,8 +31,8 @@ class ColDrop(ColumnsBasedPipelineStage):
     errors : {‘ignore’, ‘raise’}, default ‘raise’
         If ‘ignore’, suppress error and existing labels are dropped.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[8,'a'],[5,'b']], [1,2], ['num', 'char'])
     >>> pdp.ColDrop('num').apply(df)
@@ -57,19 +57,21 @@ class ColDrop(ColumnsBasedPipelineStage):
         super_kwargs['none_columns'] = 'error'
         super().__init__(**super_kwargs)
 
-    def _prec(self, df: pandas.DataFrame) -> bool:
+    def _prec(self, X: pandas.DataFrame) -> bool:
         if self._errors != 'ignore':
-            return super()._prec(df)
+            return super()._prec(X)
         return True
 
-    def _post(self, df: pandas.DataFrame) -> bool:
-        return self._post_cond(df)
+    def _post(self, X: pandas.DataFrame) -> bool:
+        return self._post_cond(X)
 
     def _transformation(
-        self, df: pandas.DataFrame, verbose: bool, fit: bool,
+        self, X: pandas.DataFrame, verbose: bool, fit: bool,
     ) -> pandas.DataFrame:
-        return df.drop(
-            self._get_columns(df, fit=fit), axis=1, errors=self._errors)
+        to_drop = self._get_columns(X, fit=fit)
+        if verbose:
+            print(f'Dropping columns {_list_str(to_drop)}')
+        return X.drop(to_drop, axis=1, errors=self._errors)
 
 
 class ValDrop(ColumnsBasedPipelineStage):
@@ -90,8 +92,8 @@ class ValDrop(ColumnsBasedPipelineStage):
         callable returning a labels iterable from an input pandas.DataFrame.
         See `pdpipe.cq`. Optional. By default no columns are excluded.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1,4],[4,5],[18,11]], [1,2,3], ['a','b'])
     >>> pdp.ValDrop([4], 'a').apply(df)
@@ -120,16 +122,16 @@ class ValDrop(ColumnsBasedPipelineStage):
         super().__init__(**super_kwargs)
 
     def _transformation(
-        self, df: pandas.DataFrame, verbose: bool, fit: bool,
+        self, X: pandas.DataFrame, verbose: bool, fit: bool,
     ) -> pandas.DataFrame:
-        inter_df = df
-        before_count = len(inter_df)
-        columns_to_check = self._get_columns(df, fit=fit)
+        inter_X = X
+        before_count = len(inter_X)
+        columns_to_check = self._get_columns(X, fit=fit)
         for col in columns_to_check:
-            inter_df = inter_df[~inter_df[col].isin(self._values)]
+            inter_X = inter_X[~inter_X[col].isin(self._values)]
         if verbose:
-            print(f"{before_count - len(inter_df)} rows dropped.")
-        return inter_df
+            print(f"{before_count - len(inter_X)} rows dropped.")
+        return inter_X
 
 
 class ValKeep(ColumnsBasedPipelineStage):
@@ -150,8 +152,8 @@ class ValKeep(ColumnsBasedPipelineStage):
         callable returning a labels iterable from an input pandas.DataFrame.
         See `pdpipe.cq`. Optional. By default no columns are excluded.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1,4],[4,5],[5,11]], [1,2,3], ['a','b'])
     >>> pdp.ValKeep([4, 5], 'a').apply(df)
@@ -174,15 +176,15 @@ class ValKeep(ColumnsBasedPipelineStage):
         super_kwargs['none_columns'] = 'all'
         super().__init__(**super_kwargs)
 
-    def _transformation(self, df, verbose, fit):
-        inter_df = df
-        before_count = len(inter_df)
-        columns_to_check = self._get_columns(df, fit=fit)
+    def _transformation(self, X, verbose, fit):
+        inter_X = X
+        before_count = len(inter_X)
+        columns_to_check = self._get_columns(X, fit=fit)
         for col in columns_to_check:
-            inter_df = inter_df[inter_df[col].isin(self._values)]
+            inter_X = inter_X[inter_X[col].isin(self._values)]
         if verbose:
-            print(f"{before_count - len(inter_df)} rows dropped.")
-        return inter_df
+            print(f"{before_count - len(inter_X)} rows dropped.")
+        return inter_X
 
 
 class ColRename(PdPipelineStage):
@@ -193,8 +195,8 @@ class ColRename(PdPipelineStage):
     rename_mapper : dict-like or callable
         Maps old column names to new ones.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[8,'a'],[5,'b']], [1,2], ['num', 'char'])
     >>> pdp.ColRename({'num': 'len', 'char': 'initial'}).apply(df)
@@ -246,11 +248,11 @@ class ColRename(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
-        return self._tprec(df)
+    def _prec(self, X):
+        return self._tprec(X)
 
-    def _transform(self, df, verbose):
-        return df.rename(columns=self._rename_mapper)
+    def _transform(self, X, verbose):
+        return X.rename(columns=self._rename_mapper)
 
 
 class DropNa(PdPipelineStage):
@@ -258,8 +260,8 @@ class DropNa(PdPipelineStage):
 
     Supports all parameter supported by pandas.dropna function.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1,4],[4,None],[1,11]], [1,2,3], ['a','b'])
     >>> pdp.DropNa().apply(df)
@@ -273,7 +275,7 @@ class DropNa(PdPipelineStage):
 
     def __init__(self, **kwargs):
         common = set(kwargs.keys()).intersection(DropNa._DROPNA_KWARGS)
-        self.dropna_kwargs = {key: kwargs.pop(key) for key in common}
+        self._dropna_kwargs = {key: kwargs.pop(key) for key in common}
         super_kwargs = {
             'exmsg': DropNa._DEF_DROPNA_EXC_MSG,
             'desc': "Drops null values."
@@ -281,19 +283,19 @@ class DropNa(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
+    def _prec(self, X):
         return True
 
-    def _transform(self, df, verbose):
-        before_count = len(df)
-        ncols_before = len(df.columns)
-        inter_df = df.dropna(**self.dropna_kwargs)
+    def _transform(self, X, verbose):
+        before_count = len(X)
+        ncols_before = len(X.columns)
+        inter_X = X.dropna(**self._dropna_kwargs)
         if verbose:
             print(
-                f"{before_count - len(inter_df)} rows, "
-                f"{ncols_before - len(inter_df.columns)} columns dropeed"
+                f"{before_count - len(inter_X)} rows, "
+                f"{ncols_before - len(inter_X.columns)} columns dropeed"
             )
-        return inter_df
+        return inter_X
 
 
 class SetIndex(PdPipelineStage):
@@ -302,8 +304,8 @@ class SetIndex(PdPipelineStage):
     Supports all parameter supported by pandas.set_index function except for
     `inplace`.
 
-    Example
-    -------
+    Examples
+    --------
     >> import pandas as pd; import pdpipe as pdp;
     >> df = pd.DataFrame([[1,4],[3, 11]], [1,2], ['a','b'])
     >> pdp.SetIndex('a').apply(df)
@@ -319,8 +321,8 @@ class SetIndex(PdPipelineStage):
 
     def __init__(self, keys, **kwargs):
         common = set(kwargs.keys()).intersection(SetIndex._SETINDEX_KWARGS)
-        self.setindex_kwargs = {key: kwargs.pop(key) for key in common}
-        self.keys = keys
+        self._setindex_kwargs = {key: kwargs.pop(key) for key in common}
+        self._keys = keys
         if hasattr(keys, '__iter__') and not isinstance(keys, str):
             _tprec = cond.HasAllColumns(list(keys))
         else:
@@ -333,11 +335,11 @@ class SetIndex(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
-        return self._tprec(df)
+    def _prec(self, X):
+        return self._tprec(X)
 
-    def _transform(self, df, verbose):
-        return df.set_index(keys=self.keys, **self.setindex_kwargs)
+    def _transform(self, X, verbose):
+        return X.set_index(keys=self._keys, **self._setindex_kwargs)
 
 
 class FreqDrop(PdPipelineStage):
@@ -350,8 +352,8 @@ class FreqDrop(PdPipelineStage):
     column : str
         The name of the colum to check for the given value frequency.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1,4],[4,5],[1,11]], [1,2,3], ['a','b'])
     >>> pdp.FreqDrop(2, 'a').apply(df)
@@ -375,18 +377,18 @@ class FreqDrop(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
-        return self._column in df.columns
+    def _prec(self, X):
+        return self._column in X.columns
 
-    def _transform(self, df, verbose):
-        inter_df = df
-        before_count = len(inter_df)
-        valcount = df[self._column].value_counts()
+    def _transform(self, X, verbose):
+        inter_X = X
+        before_count = len(inter_X)
+        valcount = X[self._column].value_counts()
         to_drop = valcount[valcount < self._threshold].index
-        inter_df = inter_df[~inter_df[self._column].isin(to_drop)]
+        inter_X = inter_X[~inter_X[self._column].isin(to_drop)]
         if verbose:
-            print(f"{before_count - len(inter_df)} rows dropped.")
-        return inter_df
+            print(f"{before_count - len(inter_X)} rows dropped.")
+        return inter_X
 
 
 class ColReorder(PdPipelineStage):
@@ -399,8 +401,8 @@ class ColReorder(PdPipelineStage):
         Columns not included in the mapping will maintain their relative
         positions over the non-mapped colums.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[8,4,3,7]], columns=['a', 'b', 'c', 'd'])
     >>> pdp.ColReorder({'b': 0, 'c': 3}).apply(df)
@@ -421,11 +423,11 @@ class ColReorder(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
-        return set(self._col_to_pos.keys()).issubset(df.columns)
+    def _prec(self, X):
+        return set(self._col_to_pos.keys()).issubset(X.columns)
 
-    def _transform(self, df, verbose):
-        cols = df.columns
+    def _transform(self, X, verbose):
+        cols = X.columns
         map_cols = list(self._col_to_pos.keys())
         non_map_cols = deque(x for x in cols if x not in map_cols)
         new_columns = []
@@ -435,7 +437,7 @@ class ColReorder(PdPipelineStage):
                     new_columns.append(self._pos_to_col[pos])
                 else:
                     new_columns.append(non_map_cols.popleft())
-            return df[new_columns]
+            return X[new_columns]
         except (IndexError):
             raise ValueError(f"Bad positions mapping given: {new_columns}")
 
@@ -473,8 +475,8 @@ class RowDrop(ColumnsBasedPipelineStage):
         callable returning a labels iterable from an input pandas.DataFrame.
         See `pdpipe.cq`. Optional. By default no columns are excluded.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1,4],[4,5],[5,11]], [1,2,3], ['a','b'])
     >>> pdp.RowDrop([lambda x: x < 2]).apply(df)
@@ -555,15 +557,15 @@ class RowDrop(ColumnsBasedPipelineStage):
         super_kwargs['none_columns'] = 'all'
         super().__init__(**super_kwargs)
 
-    def _transformation(self, df, verbose, fit):
-        before_count = len(df)
-        columns = self._get_columns(df, fit=fit)
-        subdf = df[columns]
-        drop_index = ~subdf.apply(self._row_cond, axis=1)
-        inter_df = df[drop_index]
+    def _transformation(self, X, verbose, fit):
+        before_count = len(X)
+        columns = self._get_columns(X, fit=fit)
+        subX = X[columns]
+        drop_index = ~subX.apply(self._row_cond, axis=1)
+        inter_X = X[drop_index]
         if verbose:
-            print(f"{before_count - len(inter_df)} rows dropped.")
-        return inter_df
+            print(f"{before_count - len(inter_X)} rows dropped.")
+        return inter_X
 
 
 class Schematize(PdPipelineStage):
@@ -575,8 +577,8 @@ class Schematize(PdPipelineStage):
         The dataframe schema to enforce on input dataframes. If set to None,
         the schema is learned in fit time and applied in subsequent transforms.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[2, 4, 8],[3, 6, 9]], [1, 2], ['a', 'b', 'c'])
     >>> pdp.Schematize(['a', 'c']).apply(df)
@@ -591,7 +593,7 @@ class Schematize(PdPipelineStage):
 
     def __init__(
         self,
-        columns: Optional[List[object]],
+        columns: Optional[List[object]] = None,
         **kwargs: object,
     ) -> None:
         if columns is None:
@@ -618,22 +620,22 @@ class Schematize(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df: pandas.DataFrame) -> bool:
-        if self._adaptive and not self.is_fitted:
+    def _prec(self, X: pandas.DataFrame) -> bool:
+        if self._adaptive and self._is_being_fitted:
             return True
-        return set(self._columns).issubset(df.columns)
+        return set(self._columns).issubset(X.columns)
 
     def _transform(
-            self, df: pandas.DataFrame, verbose=None) -> pandas.DataFrame:
-        return df[self._columns]
+            self, X: pandas.DataFrame, verbose=None) -> pandas.DataFrame:
+        return X[self._columns]
 
     def _fit_transform(
-            self, df: pandas.DataFrame, verbose=None) -> pandas.DataFrame:
+            self, X: pandas.DataFrame, verbose=None) -> pandas.DataFrame:
         if self._adaptive:
-            self._columns = df.columns
+            self._columns = X.columns
             self.is_fitted = True
-            return df
-        return df[self._columns]
+            return X
+        return X[self._columns]
 
 
 class DropDuplicates(ColumnsBasedPipelineStage):
@@ -669,12 +671,12 @@ class DropDuplicates(ColumnsBasedPipelineStage):
         super_kwargs['none_columns'] = 'all'
         super().__init__(**super_kwargs)
 
-    def _transformation(self, df, verbose, fit):
-        columns = self._get_columns(df, fit=fit)
-        inter_df = df.drop_duplicates(subset=columns)
+    def _transformation(self, X, verbose, fit):
+        columns = self._get_columns(X, fit=fit)
+        inter_X = X.drop_duplicates(subset=columns)
         if verbose:
-            print(f"{len(df) - len(inter_df)} rows dropped.")
-        return inter_df
+            print(f"{len(X) - len(inter_X)} rows dropped.")
+        return inter_X
 
 
 class ColumnDtypeEnforcer(PdPipelineStage):
@@ -697,8 +699,8 @@ class ColumnDtypeEnforcer(PdPipelineStage):
         - raise : allow exceptions to be raised
         - ignore : suppress exceptions. On error return original object.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[8,'a'],[5,'b']], [1,2], ['num', 'initial'])
     >>> pdp.ColumnDtypeEnforcer({'num': float}).apply(df)
@@ -706,7 +708,7 @@ class ColumnDtypeEnforcer(PdPipelineStage):
     1  8.0       a
     2  5.0       b
 
-    >>> pdp.ColumnDtypeEnforcer({pdp.cq.StartWith('n'): float}).apply(df)
+    >>> pdp.ColumnDtypeEnforcer({pdp.cq.StartsWith('n'): float}).apply(df)
        num initial
     1  8.0       a
     2  5.0       b
@@ -746,31 +748,40 @@ class ColumnDtypeEnforcer(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _col_to_dtype_from_df(self, df: pandas.DataFrame) -> Dict:
+    def _col_to_dtype_from_X(self, X: pandas.DataFrame) -> Dict:
         try:
             return self._column_to_dtype
         except AttributeError:
             column_to_dtype = {}
             for k, dtype in self._dynamic_column_to_dtype.items():
                 try:
+                    columns = []
+                    try:
+                        if k._fittable and self._is_being_fitted:
+                            columns = k.fit_transform(X)
+                    except AttributeError:
+                        # k is not a ColumnQualifier
+                        columns = k(X)
                     column_to_dtype.update({
                         lbl: dtype
-                        for lbl in k(df)
+                        for lbl in columns
                     })
                 except TypeError:  # k is not a callable
                     column_to_dtype[k] = dtype
             return column_to_dtype
 
-    def _prec(self, df: pandas.DataFrame) -> bool:
-        return self._tprec(df)
+    def _prec(self, X: pandas.DataFrame) -> bool:
+        if self._is_being_fitted:
+            return self._tprec.fit_transform(X)
+        return self._tprec(X)
 
     def _transform(
         self,
-        df: pandas.DataFrame,
+        X: pandas.DataFrame,
         verbose: bool,
     ) -> pandas.DataFrame:
-        lbl_to_dtype = self._col_to_dtype_from_df(df)
-        return df.astype(
+        lbl_to_dtype = self._col_to_dtype_from_X(X)
+        return X.astype(
             dtype=lbl_to_dtype,
             copy=True,
             errors=self._errors,
@@ -807,8 +818,8 @@ class ConditionValidator(PdPipelineStage):
         application was called with `verbose=True`, and pipeline application
         continues. Any other value is interpreted as 'raise'.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1,4],[4,None],[1,11]], [1,2,3], ['a','b'])
     >>> pdp.ConditionValidator(lambda df: len(df.columns) == 5).apply(df)
@@ -844,14 +855,14 @@ class ConditionValidator(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
+    def _prec(self, X):
         return True
 
-    def _transform(self, df, verbose):
+    def _transform(self, X, verbose):
         results = []
         for cond_obj in self._conditions:
             try:
-                res = cond_obj(df)
+                res = cond_obj(X)
             except Exception as e:
                 raise ValueError(
                     f"Supplied condition raised a {e} exception when applied "
@@ -868,7 +879,7 @@ class ConditionValidator(PdPipelineStage):
             raise FailedConditionError(
                 "ConditionValidator stage failed; some conditions did not hold"
                 " for the input dataframe!")
-        return df
+        return X
 
 
 class ApplicationContextEnricher(PdPipelineStage):
@@ -886,7 +897,7 @@ class ApplicationContextEnricher(PdPipelineStage):
 
     Mappings are evaluated in the order they are passed to the constructor.
 
-    For example, `ApplicationContextEnricher(suma=lambda df: df['a'].sum())'
+    For example, `ApplicationContextEnricher(suma=lambda df: df['a'].sum())`
     will add the sum of the 'a' column to the application context keys under
     the key 'suma'. Lated stages can then access the value of 'suma' with
     `self.application_context['suma']`.
@@ -924,18 +935,18 @@ class ApplicationContextEnricher(PdPipelineStage):
         super_kwargs.update(**init_kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
+    def _prec(self, X):
         return True
 
-    def _transform(self, df, verbose):
+    def _transform(self, X, verbose):
         for k, v in self._enrichments.items():
             if callable(v):
                 try:
                     self.application_context[k] = v(
-                        df, application_context=self.application_context)
+                        X, application_context=self.application_context)
                 except TypeError:
                     try:
-                        self.application_context[k] = v(df)
+                        self.application_context[k] = v(X)
                     except Exception as e:
                         raise ValueError((
                             f"Supplied enrichment function raised a {e} "
@@ -943,4 +954,4 @@ class ApplicationContextEnricher(PdPipelineStage):
                         )) from e
             else:
                 self.application_context[k] = v
-        return df
+        return X
