@@ -8,7 +8,8 @@ from typing import Union, Tuple, Optional, Dict, Callable
 import numpy as np
 import pandas as pd
 import sortedcontainers as sc
-import tqdm
+# import tqdm
+from tqdm.autonotebook import tqdm
 
 from pdpipe.core import (
     PdpApplicationContext,
@@ -48,8 +49,8 @@ class Bin(PdPipelineStage):
     drop : bool, default True
         If set to True, the source columns are dropped after being binned.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[-3],[4],[5],[9]], [1,2,3,4], ['speed'])
     >>> pdp.Bin({'speed': [5]}, drop=False).apply(df)
@@ -92,8 +93,8 @@ class Bin(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
-        return set(self._bin_map.keys()).issubset(df.columns)
+    def _prec(self, X):
+        return set(self._bin_map.keys()).issubset(X.columns)
 
     @staticmethod
     def _get_col_binner(bin_list):
@@ -116,30 +117,30 @@ class Bin(PdPipelineStage):
 
         return _col_binner
 
-    def _transform(self, df, verbose):
-        inter_df = df
+    def _transform(self, X, verbose):
+        inter_X = X
         colnames = list(self._bin_map.keys())
         if verbose:
-            colnames = tqdm.tqdm(colnames)
+            colnames = tqdm(colnames)
         for colname in colnames:
             if verbose:
                 colnames.set_description(colname)
-            source_col = df[colname]
-            loc = df.columns.get_loc(colname) + 1
+            source_col = X[colname]
+            loc = X.columns.get_loc(colname) + 1
             new_name = colname + "_bin"
             if self._drop:
-                inter_df = inter_df.drop(colname, axis=1)
+                inter_X = inter_X.drop(colname, axis=1)
                 new_name = colname
                 loc -= 1
-            inter_df = out_of_place_col_insert(
-                df=inter_df,
+            inter_X = out_of_place_col_insert(
+                X=inter_X,
                 series=source_col.apply(
                     self._get_col_binner(self._bin_map[colname])
                 ),
                 loc=loc,
                 column_name=new_name,
             )
-        return inter_df
+        return inter_X
 
 
 class OneHotEncode(ColumnsBasedPipelineStage):
@@ -174,8 +175,8 @@ class OneHotEncode(ColumnsBasedPipelineStage):
     drop : bool, default True
         If set to True, the source columns are dropped after being encoded.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([['USA'], ['UK'], ['Greece']], [1,2,3], ['Born'])
     >>> pdp.OneHotEncode().apply(df)
@@ -223,19 +224,19 @@ class OneHotEncode(ColumnsBasedPipelineStage):
         super_kwargs['none_columns'] = OfDtypes(['object', 'category'])
         super().__init__(**super_kwargs)
 
-    def _transformation(self, df, verbose, fit):
+    def _transformation(self, X, verbose, fit):
         raise NotImplementedError
 
-    def _fit_transform(self, df, verbose):
-        columns_to_encode = self._get_columns(df, fit=True)
+    def _fit_transform(self, X, verbose):
+        columns_to_encode = self._get_columns(X, fit=True)
         assign_map = {}
         if verbose:
-            columns_to_encode = tqdm.tqdm(columns_to_encode)
+            columns_to_encode = tqdm(columns_to_encode)
         for colname in columns_to_encode:
             if verbose:
                 columns_to_encode.set_description(colname)
             dummies = pd.get_dummies(
-                df[colname],
+                X[colname],
                 drop_first=False,
                 dummy_na=self._dummy_na,
                 prefix=colname,
@@ -264,17 +265,17 @@ class OneHotEncode(ColumnsBasedPipelineStage):
             for column in dummies:
                 assign_map[column] = dummies[column]
 
-        inter_df = df.assign(**assign_map)
+        inter_X = X.assign(**assign_map)
         self.is_fitted = True
         if self._drop:
-            return inter_df.drop(columns_to_encode, axis=1)
-        return inter_df
+            return inter_X.drop(columns_to_encode, axis=1)
+        return inter_X
 
-    def _transform(self, df, verbose):
+    def _transform(self, X, verbose):
         assign_map = {}
-        columns_to_encode = self._get_columns(df, fit=False)
+        columns_to_encode = self._get_columns(X, fit=False)
         for colname in columns_to_encode:
-            col = df[colname]
+            col = X[colname]
             try:
                 encoder = self._encoder_map[colname]
             except KeyError:  # pragma: no cover
@@ -285,10 +286,10 @@ class OneHotEncode(ColumnsBasedPipelineStage):
             res_cols = col.astype('object').apply(encoder)
             for res_col in res_cols:
                 assign_map[res_col] = res_cols[res_col]
-        inter_df = df.assign(**assign_map)
+        inter_X = X.assign(**assign_map)
         if self._drop:
-            return inter_df.drop(columns_to_encode, axis=1)
-        return inter_df
+            return inter_X.drop(columns_to_encode, axis=1)
+        return inter_X
 
 
 class ColumnTransformer(ColumnsBasedPipelineStage):
@@ -342,7 +343,7 @@ class ColumnTransformer(ColumnsBasedPipelineStage):
     ):
         if suffix is None:  # pragma: no cover
             suffix = "_transformed"
-        self.suffix = suffix
+        self._suffix = suffix
         self._result_columns = result_columns
         if result_columns:
             self._result_columns = _interpret_columns_param(result_columns)
@@ -371,31 +372,31 @@ class ColumnTransformer(ColumnsBasedPipelineStage):
     ) -> pd.Series:
         raise NotImplementedError
 
-    def _transformation(self, df, verbose, fit):
-        columns = self._get_columns(df, fit=fit)
+    def _transformation(self, X, verbose, fit):
+        columns = self._get_columns(X, fit=fit)
         result_columns = self._result_columns
         if self._result_columns is None:
             if self._drop:
                 result_columns = columns
             else:
                 result_columns = [
-                    f'{col}{self.suffix}' for col in columns
+                    f'{col}{self._suffix}' for col in columns
                 ]
-        inter_df = df
+        inter_X = X
         for i, colname in enumerate(columns):
-            source_col = df[colname]
-            loc = df.columns.get_loc(colname) + 1
+            source_col = X[colname]
+            loc = X.columns.get_loc(colname) + 1
             new_name = result_columns[i]
             if self._drop:
-                inter_df = inter_df.drop(colname, axis=1)
+                inter_X = inter_X.drop(colname, axis=1)
                 loc -= 1
-            inter_df = out_of_place_col_insert(
-                df=inter_df,
+            inter_X = out_of_place_col_insert(
+                X=inter_X,
                 series=self._col_transform(source_col, colname),
                 loc=loc,
                 column_name=new_name,
             )
-        return inter_df
+        return inter_X
 
 
 class _AttrGetter():
@@ -471,8 +472,8 @@ class MapColVals(ColumnTransformer):
     suffix : str, default '_map'
         The suffix mapped columns gain if no new column labels are given.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> df = pd.DataFrame([[1], [3], [2]], ['UK', 'USSR', 'US'], ['Medal'])
     >>> value_map = {1: 'Gold', 2: 'Silver', 3: 'Bronze'}
@@ -559,8 +560,8 @@ class ApplyToRows(PdPipelineStage):
         returning True is used.
 
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> data = [[3, 2143], [10, 1321], [7, 1255]]
     >>> df = pd.DataFrame(data, [1,2,3], ['years', 'avg_revenue'])
@@ -613,37 +614,37 @@ class ApplyToRows(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
-        return self._prec_func(df)
+    def _prec(self, X):
+        return self._prec_func(X)
 
-    def _transform(self, df, verbose):
-        new_cols = df.apply(self._func, axis=1)
+    def _transform(self, X, verbose):
+        new_cols = X.apply(self._func, axis=1)
         if isinstance(new_cols, pd.Series):
-            loc = len(df.columns)
+            loc = len(X.columns)
             if self._follow_column:
-                loc = df.columns.get_loc(self._follow_column) + 1
+                loc = X.columns.get_loc(self._follow_column) + 1
             return out_of_place_col_insert(
-                df=df, series=new_cols, loc=loc, column_name=self._colname
+                X=X, series=new_cols, loc=loc, column_name=self._colname
             )
         if isinstance(new_cols, pd.DataFrame):
             sorted_cols = sorted(list(new_cols.columns))
             new_cols = new_cols[sorted_cols]
             if self._follow_column:
-                inter_df = df
-                loc = df.columns.get_loc(self._follow_column) + 1
+                inter_X = X
+                loc = X.columns.get_loc(self._follow_column) + 1
                 for colname in new_cols.columns:
-                    inter_df = out_of_place_col_insert(
-                        df=inter_df,
+                    inter_X = out_of_place_col_insert(
+                        X=inter_X,
                         series=new_cols[colname],
                         loc=loc,
                         column_name=colname,
                     )
                     loc += 1
-                return inter_df
+                return inter_X
             assign_map = {
                 colname: new_cols[colname] for colname in new_cols.columns
             }
-            return df.assign(**assign_map)
+            return X.assign(**assign_map)
         raise TypeError(  # pragma: no cover
             "Unexpected type generated by applying a function to a DataFrame."
             " Only Series and DataFrame are allowed."
@@ -683,8 +684,8 @@ class ApplyByCols(ColumnTransformer):
         Valid constructor parameters of superclasses are extracted and used
         on intialization.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp; import math;
     >>> data = [[3.2, "acd"], [7.2, "alk"], [12.1, "alk"]]
     >>> df = pd.DataFrame(data, [1,2,3], ["ph","lbl"])
@@ -784,8 +785,8 @@ class ColByFrameFunc(PdPipelineStage):
         by company size'. A default description is used if None is given.
 
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> data = [[3, 3], [2, 4], [1, 5]]
     >>> df = pd.DataFrame(data, [1,2,3], ["A","B"])
@@ -823,28 +824,28 @@ class ColByFrameFunc(PdPipelineStage):
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
-    def _prec(self, df):
+    def _prec(self, X):
         return True
 
-    def _transform(self, df, verbose):
-        inter_df = df
+    def _transform(self, X, verbose):
+        inter_X = X
         try:
-            new_col = self._func(df)
+            new_col = self._func(X)
         except Exception:
             raise PipelineApplicationError(
                 f"Exception raised applying function {self._func_desc} to "
                 f"dataframe by class {self.__class__}."
             )
         if self._follow_column:
-            loc = df.columns.get_loc(self._follow_column) + 1
+            loc = X.columns.get_loc(self._follow_column) + 1
         elif self._before_column:
-            loc = df.columns.get_loc(self._before_column)
+            loc = X.columns.get_loc(self._before_column)
         else:
-            loc = len(df.columns)
-        inter_df = out_of_place_col_insert(
-            df=inter_df, series=new_col, loc=loc, column_name=self._column
+            loc = len(X.columns)
+        inter_X = out_of_place_col_insert(
+            X=inter_X, series=new_col, loc=loc, column_name=self._column
         )
-        return inter_df
+        return inter_X
 
 
 class AggByCols(ColumnTransformer):
@@ -877,8 +878,8 @@ class AggByCols(ColumnTransformer):
         is None and drop is set to False. Of not given, defaults to '_agg'.
 
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp; import numpy as np;
     >>> data = [[3.2, "acd"], [7.2, "alk"], [12.1, "alk"]]
     >>> df = pd.DataFrame(data, [1,2,3], ["ph","lbl"])
@@ -957,8 +958,8 @@ class Log(ColumnsBasedPipelineStage):
         non_neg is True then that transformation is applied first, and only
         then is the column shifted by this constant.
 
-    Example
-    -------
+    Examples
+    --------
     >>> import pandas as pd; import pdpipe as pdp;
     >>> data = [[3.2, "acd"], [7.2, "alk"], [12.1, "alk"]]
     >>> df = pd.DataFrame(data, [1,2,3], ["ph","lbl"])
@@ -994,20 +995,20 @@ class Log(ColumnsBasedPipelineStage):
         super_kwargs['none_columns'] = OfDtypes([np.number])
         super().__init__(**super_kwargs)
 
-    def _transformation(self, df, verbose, fit):
+    def _transformation(self, X, verbose, fit):
         raise NotImplementedError
 
-    def _fit_transform(self, df, verbose):
-        columns_to_transform = self._get_columns(df, fit=True)
+    def _fit_transform(self, X, verbose):
+        columns_to_transform = self._get_columns(X, fit=True)
         if verbose:
-            columns_to_transform = tqdm.tqdm(columns_to_transform)
-        inter_df = df
+            columns_to_transform = tqdm(columns_to_transform)
+        inter_X = X
         for colname in columns_to_transform:
-            source_col = df[colname]
-            loc = df.columns.get_loc(colname) + 1
+            source_col = X[colname]
+            loc = X.columns.get_loc(colname) + 1
             new_name = colname + "_log"
             if self._drop:
-                inter_df = inter_df.drop(colname, axis=1)
+                inter_X = inter_X.drop(colname, axis=1)
                 new_name = colname
                 loc -= 1
             new_col = source_col
@@ -1022,29 +1023,29 @@ class Log(ColumnsBasedPipelineStage):
             if self._const_shift is not None:
                 new_col = new_col + self._const_shift
             new_col = np.log(new_col)
-            inter_df = out_of_place_col_insert(
-                df=inter_df, series=new_col, loc=loc, column_name=new_name
+            inter_X = out_of_place_col_insert(
+                X=inter_X, series=new_col, loc=loc, column_name=new_name
             )
         self.is_fitted = True
-        return inter_df
+        return inter_X
 
-    def _transform(self, df, verbose):
-        inter_df = df
-        columns_to_transform = self._get_columns(df, fit=False)
+    def _transform(self, X, verbose):
+        inter_X = X
+        columns_to_transform = self._get_columns(X, fit=False)
         if verbose:
-            columns_to_transform = tqdm.tqdm(columns_to_transform)
+            columns_to_transform = tqdm(columns_to_transform)
         for colname in columns_to_transform:
             try:
-                source_col = df[colname]
+                source_col = X[colname]
             except KeyError:  # pragma: no cover
                 raise PipelineApplicationError((
                     "Missig column {} when applying a fitted "
                     "Log pipeline stage by class {} !").format(
                         colname, self.__class__))
-            loc = df.columns.get_loc(colname) + 1
+            loc = X.columns.get_loc(colname) + 1
             new_name = colname + "_log"
             if self._drop:
-                inter_df = inter_df.drop(colname, axis=1)
+                inter_X = inter_X.drop(colname, axis=1)
                 new_name = colname
                 loc -= 1
             new_col = source_col
@@ -1061,7 +1062,7 @@ class Log(ColumnsBasedPipelineStage):
             if self._const_shift is not None:
                 new_col = new_col + self._const_shift
             new_col = np.log(new_col)
-            inter_df = out_of_place_col_insert(
-                df=inter_df, series=new_col, loc=loc, column_name=new_name
+            inter_X = out_of_place_col_insert(
+                X=inter_X, series=new_col, loc=loc, column_name=new_name
             )
-        return inter_df
+        return inter_X
